//
//  SCSAnnotationsView.swift
//  SciChartSwiftDemo
//
//  Created by Mykola Hrybeniuk on 6/6/16.
//  Copyright © 2016 SciChart Ltd. All rights reserved.
//

import Foundation
import SciChart

class SCSAnnotationsChartView: SCSBaseChartView {
    
    // MARK: Overrided Functions
    
    override func completeConfiguration() {
        super.completeConfiguration()
        addAxis()
        addDefaultModifiers()
        setupAnnotations()
        chartSurface.invalidateElement()
    }
    
    // MARK: Private Functions
    
    private func addAxis() {
        
        let axisStyle = generateDefaultAxisStyle()
        
        chartSurface.attachAxis(SCSFactoryAxis.createDefaultNumericAxis(withAxisStyle: axisStyle, axisId: axisXId), isXAxis: true)
        chartSurface.attachAxis(SCSFactoryAxis.createDefaultNumericAxis(withAxisStyle: axisStyle, axisId: axisYId), isXAxis: false)
        
    }
    
    override func addDefaultModifiers() {
        
        let xAxisDragmodifier = SCIXAxisDragModifier()
        xAxisDragmodifier.modifierName = xAxisDragModifierName
        xAxisDragmodifier.axisId = axisXId
        xAxisDragmodifier.dragMode = .Scale
        xAxisDragmodifier.clipModeX = .None
        
        let yAxisDragmodifier = SCIYAxisDragModifier()
        yAxisDragmodifier.modifierName = yAxisDragModifierName
        yAxisDragmodifier.axisId = axisYId
        yAxisDragmodifier.dragMode = .Pan
        
        let extendZoomModifier = SCIZoomExtentsModifier()
        extendZoomModifier.modifierName = extendZoomModifierName
        
        let pinchZoomModifier = SCIPinchZoomModifier()
        pinchZoomModifier.modifierName = pinchZoomModifierName
        
        let zoomPanModifier = SCIZoomPanModifier()
        zoomPanModifier.clipModeX = .None;
        
        let groupModifier = SCIModifierGroup(childModifiers: [xAxisDragmodifier, yAxisDragmodifier, pinchZoomModifier, extendZoomModifier, zoomPanModifier])
        
        chartSurface.chartModifier = groupModifier
    }
    
     private func setupAnnotations() {
        // Box bound to chart surface
        let boxBlue = SCIBoxAnnotation()
        boxBlue.xAxisId = "xAxis"
        boxBlue.yAxisId = "yAxis"
        boxBlue.coordMode = .Absolute
        boxBlue.xStart = SCIGeneric(4)
        boxBlue.yStart = SCIGeneric(8)
        boxBlue.xEnd = SCIGeneric(7)
        boxBlue.yEnd = SCIGeneric(4)
        boxBlue.isEnabled = false
        boxBlue.style.fillBrush = SCIBrushSolid(colorCode: 0x30FF7000)
        
        // Box bound to screen position
        let boxRed = SCIBoxAnnotation()
        boxRed.xAxisId = "xAxis"
        boxRed.yAxisId = "yAxis"
        boxRed.coordMode = .Relative
        boxRed.xStart = SCIGeneric(0.25)
        boxRed.yStart = SCIGeneric(0.25)
        boxRed.xEnd = SCIGeneric(0.5)
        boxRed.yEnd = SCIGeneric(0.5)
        boxRed.isEnabled = false
        boxRed.style.fillBrush = SCIBrushSolid(colorCode: 0x301010FF)
        boxRed.style.borderPen = SCIPenSolid(colorCode: 0xFF0000FF, width: 2)
        // line bound to position on screen
        let lineAnnotationRelative = SCILineAnnotation()
        lineAnnotationRelative.xAxisId = "xAxis"
        lineAnnotationRelative.yAxisId = "yAxis"
        lineAnnotationRelative.coordMode = .Relative
        lineAnnotationRelative.xStart = SCIGeneric(0.1)
        lineAnnotationRelative.yStart = SCIGeneric(0.1)
        lineAnnotationRelative.xEnd = SCIGeneric(0.9)
        lineAnnotationRelative.yEnd = SCIGeneric(0.1)
        lineAnnotationRelative.style.linePen = SCIPenSolid(colorCode: 0xFF0000FF, width: 2)
        
        // line bound to position on chart
        let lineAnnotationAbsolute = SCILineAnnotation()
        lineAnnotationAbsolute.xAxisId = "xAxis"
        lineAnnotationAbsolute.yAxisId = "yAxis"
        lineAnnotationAbsolute.coordMode = .Absolute
        lineAnnotationAbsolute.xStart = SCIGeneric(2)
        lineAnnotationAbsolute.yStart = SCIGeneric(2)
        lineAnnotationAbsolute.xEnd = SCIGeneric(5)
        lineAnnotationAbsolute.yEnd = SCIGeneric(6)
        lineAnnotationAbsolute.style.linePen = SCIPenSolid(colorCode: 0xFF00FF00, width: 2)
        
        // line with X position bound to chart and Y to screen
        let lineAnnotationAbsoluteX = SCILineAnnotation()
        lineAnnotationAbsoluteX.xAxisId = "xAxis"
        lineAnnotationAbsoluteX.yAxisId = "yAxis"
        lineAnnotationAbsoluteX.coordMode = .RelativeY
        lineAnnotationAbsoluteX.xStart = SCIGeneric(1)
        lineAnnotationAbsoluteX.yStart = SCIGeneric(0.05)
        lineAnnotationAbsoluteX.xEnd = SCIGeneric(1)
        lineAnnotationAbsoluteX.yEnd = SCIGeneric(0.95)
        lineAnnotationAbsoluteX.style.linePen = SCIPenSolid(colorCode: 0xFFFF0000, width: 2)
        
        // axis marker bound to chart surface
        let xMarker = SCIAxisMarkerAnnotation()
        xMarker.coordMode = .Absolute
        xMarker.xAxisId = "xAxis"
        xMarker.position = SCIGeneric(2.5)
        xMarker.style.markerLinePen = SCIPenSolid(colorCode: 0xAFFFFF00, width: 1)
        xMarker.style.backgroundColor = UIColor.fromColorCode(0xFFCFCF30)
        xMarker.style.textStyle.transform = CGAffineTransformMakeRotation(CGFloat(M_PI_2))
        
        // axis marker bound to screen
        let yMarker = SCIAxisMarkerAnnotation()
        yMarker.coordMode = .Relative
        yMarker.yAxisId = "yAxis"
        yMarker.position = SCIGeneric(0.5)
        yMarker.style.markerLinePen = SCIPenSolid(colorCode: 0xA00000FF, width: 1)
        yMarker.style.backgroundColor = UIColor.fromColorCode(0xFF0000A0)
        yMarker.style.textStyle.fontSize = 14
        yMarker.style.textStyle.fontName = "Helvetica-Bold"
        yMarker.style.textStyle.color = UIColor.whiteColor()
        // axis marker annotation text is formated by axis as cursor text
        chartSurface.yAxes.getAxisById("yAxis").cursorTextFormatting = "%.2f"
        
        // text annotation
        let textAnnotation = SCITextAnnotation()
        textAnnotation.xAxisId = "xAxis"
        textAnnotation.yAxisId = "yAxis"
        textAnnotation.coordMode = .Relative
        textAnnotation.xPosition = SCIGeneric(0.7)
        textAnnotation.yPosition = SCIGeneric(0.5)
        
        textAnnotation.text = "Red box: position bound to screen\n" +
            "Blue box: position bound to chart surface\n" +
            "Red line: bound to screen\n" +
            "Green line: bound to surface\n" +
            "Blue line: X bound to chart, Y bound to screen\n" +
        "All annotations but axis markers are interactive"
        textAnnotation.style.textStyle.fontSize = 18
        textAnnotation.style.textColor = UIColor.whiteColor()
        textAnnotation.style.backgroundColor = UIColor.clearColor()
        
        let annotationGroup = SCIAnnotationGroup(childAnnotations: [boxBlue, boxRed, lineAnnotationRelative, lineAnnotationAbsolute, lineAnnotationAbsoluteX, textAnnotation, xMarker, yMarker])
        
        chartSurface.annotation = annotationGroup
    }
    
    
}
