//
//  SCSCollumnDrillDownView.swift
//  SciChartSwiftDemo
//
//  Created by Admin on 9/30/16.
//  Copyright © 2016 SciChart Ltd. All rights reserved.
//

import UIKit
import SciChart

// MARK: Custom modifier

class ColumnHitTest: SCIGestureModifier {
    var tapedAtIndex : SCIActionBlock_Pint! = nil
    var doubleTaped : SCIActionBlock! = nil
    
    var location = CGPoint.zero  // position of tap gesture
    var index : Int32 = 0
    
    override init() {
        super.init()
    }
    
    override func onTapGesture (gesture: UITapGestureRecognizer!, at view: UIView!) -> Bool {
        let location : CGPoint = gesture.locationInView(view)
        let rs : SCIRenderSurface = self.parentSurface.renderSurface
        if (!rs.isPointWithinBounds(location)) { return false }
        if (gesture.state == .Ended) {
            // save location of touch
            self.location = location
            let parent : SCIChartSurfaceProtocol = self.parentSurface
            let series : NSArray = parent.renderableSeries()
            let surface : SCIRenderSurface = parent.renderSurface
            let actualLocation : CGPoint = surface.pointInChartFrame(location)
            let count : Int = series.count
            // check every renderable series for hit
            for i in 0..<count {
                let rSeries : SCIRenderableSeries = series[i] as! SCIRenderableSeries
                let data : SCIRenderPassDataProtocol = rSeries.currentRenderPassData
                let hitTest : SCIHitTestProvider = rSeries.hitTestProvider() // get hit test tools
                // hit test verticaly: check if vertical projection through touch location crosses chart
                let hitTestResult : HitTestResult = hitTest.hitTestVerticalAtX(Double(actualLocation.x), y: Double(actualLocation.y), radius: 5, onData: data)
                if (hitTestResult.match) {
                    // if hit is registered on series
                    // get values at closest point to hit test position
                    self.index = hitTestResult.index
                    if tapedAtIndex != nil {
                        tapedAtIndex(index)
                    }
                }
            }
        }
        return true
    }
    
    override func onDoubleTapGesture (gesture: UITapGestureRecognizer!, at view: UIView!) -> Bool {
        let location : CGPoint = gesture.locationInView(view)
        let rs : SCIRenderSurface = self.parentSurface.renderSurface
        if (!rs.isPointWithinBounds(location)) { return false }
        if doubleTaped != nil {
            doubleTaped()
        }
        return true
    }
}

// MARK: Palette provider

class ColumnDrillDownPalette: SCIPaletteProvider {
    var styles = [SCIStyle!]()
    
    override init() {
        super.init()
        self.styles = [SCIStyle!]()
    }
    
    func addStyle(style: SCIStyle) {
        styles.append(style)
    }
    
    override func styleForX(x: Double, y: Double, index: Int32) -> SCIStyle! {
        if (index > Int32(styles.count)) {
            return nil
        }
        return styles[Int(index)]
    }
}

// MARK: Chart surface

class SCSColumnDrillDownView: SCSBaseChartView {
    
    var firstData: SCIXyDataSeries! = nil
    var secondData: SCIXyDataSeries! = nil
    var thirdData: SCIXyDataSeries! = nil
    var totalData: SCIXyDataSeries! = nil
    var firstColumn: SCIFastColumnRenderableSeries! = nil
    var secondColumn: SCIFastColumnRenderableSeries! = nil
    var thirdColumn: SCIFastColumnRenderableSeries! = nil
    var totalColumn: SCIFastColumnRenderableSeries! = nil
    var _isShowingTotal = false
    
    override init(frame: CGRect) {
        super.init(frame: frame)
        completeConfiguration()
    }
    
    required init?(coder aDecoder: NSCoder) {
        super.init(coder: aDecoder)
    }
    
    // MARK: Overrided Functions
    
    override func awakeFromNib() {
        super.awakeFromNib()
        completeConfiguration()
    }
    
    // MARK: Private Functions
    
    override func completeConfiguration() {
        super.completeConfiguration()
        addAxes()
        addDefaultModifiers()
        createData()
        initializeSurfaceRenderableSeries()
        showTotal()
    }
    
    private func addAxes() {
        
        let axisStyle = generateDefaultAxisStyle()
        chartSurface.attachAxis(SCSFactoryAxis.createCategoryNumericAxis(withAxisStyle: axisStyle, axisId: axisXId), isXAxis: true)
        chartSurface.attachAxis(SCSFactoryAxis.createDefaultNumericAxis(withAxisStyle: axisStyle, axisId: axisYId), isXAxis: false)
        addDefaultModifiers()
        
    }
    
    override func addDefaultModifiers() {
        
        let xAxisDragmodifier = SCIXAxisDragModifier()
        xAxisDragmodifier.axisId = "xAxis"
        xAxisDragmodifier.dragMode = .Scale
        xAxisDragmodifier.clipModeX = .None
        
        let yAxisDragmodifier = SCIYAxisDragModifier()
        yAxisDragmodifier.axisId = "yAxis"
        yAxisDragmodifier.dragMode = .Pan
        
        let pinchZoomModifier = SCIPinchZoomModifier()
        
        let drillDownModifier = ColumnHitTest()
        weak var wSelf = self
        drillDownModifier.doubleTaped = {() -> Void in
            wSelf!.showTotal()
        }
        drillDownModifier.tapedAtIndex = {(index: Int32) -> Void in
            wSelf!.showDetailedChart(index)
        }
        
        let groupModifier = SCIModifierGroup(childModifiers: [xAxisDragmodifier, yAxisDragmodifier, pinchZoomModifier, drillDownModifier])
        
        chartSurface.chartModifier = groupModifier
    }
 
    func createData() {
        self.totalData = SCIXyDataSeries(categoryXType: .Int32, YType: .Int32)
        do {
            self.firstData = SCIXyDataSeries(categoryXType: .Int32, YType: .Int32)
            var total : Int32 = 0
            let count = arc4random_uniform(5) + 3
            for i in 0..<count {
                let value : Int32 = Int32(arc4random_uniform(5 + i) + 1)
                total += value
                firstData.appendX(SCIGeneric(i), y: SCIGeneric(value))
            }
            totalData.appendX(SCIGeneric(1), y: SCIGeneric(total))
        }
        do {
            self.secondData = SCIXyDataSeries(categoryXType: .Int32, YType: .Int32)
            var total : Int32 = 0
            let count = arc4random_uniform(5) + 3
            for i in 0..<count {
                let value : Int32 = Int32(arc4random_uniform(5 + i) + 1)
                total += value
                secondData.appendX(SCIGeneric(i), y: SCIGeneric(value))
            }
            totalData.appendX(SCIGeneric(2), y: SCIGeneric(total))
        }
        do {
            self.thirdData = SCIXyDataSeries(categoryXType: .Int32, YType: .Int32)
            var total : Int32 = 0
            let count = arc4random_uniform(5) + 3
            for i in 0..<count {
                let value : Int32 = Int32(arc4random_uniform(5 + i) + 1)
                total += value
                thirdData.appendX(SCIGeneric(i), y: SCIGeneric(value))
            }
            totalData.appendX(SCIGeneric(3), y: SCIGeneric(total))
        }
    }
    
    func initializeSurfaceRenderableSeries() {
        self.firstColumn = SCIFastColumnRenderableSeries()
        self.firstColumn.style.fillBrush = SCIBrushLinearGradient(colorCodeStart: 0xFFFFFF00, finish: 0xA0FFFF00, direction: .Vertical)
        self.firstColumn.style.drawBorders = false
        self.firstColumn.xAxisId = "xAxis"
        self.firstColumn.yAxisId = "yAxis"
        self.firstColumn.dataSeries = firstData
        
        self.secondColumn = SCIFastColumnRenderableSeries()
        self.secondColumn.style.fillBrush = SCIBrushLinearGradient(colorCodeStart: 0xFF00FF00, finish: 0xA000FF00, direction: .Vertical)
        self.secondColumn.style.drawBorders = false
        self.secondColumn.xAxisId = "xAxis"
        self.secondColumn.yAxisId = "yAxis"
        self.secondColumn.dataSeries = secondData
        
        self.thirdColumn = SCIFastColumnRenderableSeries()
        self.thirdColumn.style.fillBrush = SCIBrushLinearGradient(colorCodeStart: 0xFF0000FF, finish: 0xA00000FF, direction: .Vertical)
        self.thirdColumn.style.drawBorders = false
        self.thirdColumn.xAxisId = "xAxis"
        self.thirdColumn.yAxisId = "yAxis"
        self.thirdColumn.dataSeries = thirdData
        
        self.totalColumn = SCIFastColumnRenderableSeries()
        self.totalColumn.style.fillBrush = SCIBrushLinearGradient(colorCodeStart: 0xFF505050, finish: 0xA505050, direction: .Vertical)
        self.totalColumn.style.drawBorders = false
        self.totalColumn.xAxisId = "xAxis"
        self.totalColumn.yAxisId = "yAxis"
        self.totalColumn.dataSeries = totalData
        
        let palette = ColumnDrillDownPalette()
        palette.addStyle(firstColumn.style)
        palette.addStyle(secondColumn.style)
        palette.addStyle(thirdColumn.style)
        self.totalColumn.paletteProvider = palette
    }
    
    func showTotal() {
        _isShowingTotal = true
        chartSurface.clearSeries()
        chartSurface.attachRenderableSeries(totalColumn)
        chartSurface.viewportManager.zoomExtents()
        chartSurface.invalidateElement()
    }
    
    func showDetailedChart(index: Int32) {
        if (!_isShowingTotal) {
            return
        }
        if index == 0 {
            self.showFirst()
        }
        else if index == 1 {
            self.showSecond()
        }
        else if index == 2 {
            self.showThird()
        }
        
    }
    
    func showFirst() {
        _isShowingTotal = false
        chartSurface.clearSeries()
        chartSurface.attachRenderableSeries(firstColumn)
        chartSurface.viewportManager.zoomExtents()
        chartSurface.invalidateElement()
    }
    
    func showSecond() {
        _isShowingTotal = false
        chartSurface.clearSeries()
        chartSurface.attachRenderableSeries(secondColumn)
        chartSurface.viewportManager.zoomExtents()
        chartSurface.invalidateElement()
    }
    
    func showThird() {
        _isShowingTotal = false
        chartSurface.clearSeries()
        chartSurface.attachRenderableSeries(thirdColumn)
        chartSurface.viewportManager.zoomExtents()
        chartSurface.invalidateElement()
    }
}
