//
//  SCSErrorBarsChartView.swift
//  SciChartSwiftDemo
//
//  Created by Hrybenuik Mykola on 9/20/16.
//  Copyright © 2016 SciChart Ltd. All rights reserved.
//

import Foundation
import SciChart

class SCSErrorBarsChartView: SCSBaseChartView {
    
    // MARK: Overrided Functions
    
    override func completeConfiguration() {
        super.completeConfiguration()
        addAxes()
        addSeries()
    }
    
    // MARK: Private Functions
    
    private func addAxes() {
        
        let axisStyle = generateDefaultAxisStyle()
        chartSurface.attachAxis(SCSFactoryAxis.createDefaultNumericAxis(withAxisStyle: axisStyle, axisId: axisXId), isXAxis: true)
        chartSurface.attachAxis(SCSFactoryAxis.createDefaultNumericAxis(withAxisStyle: axisStyle, axisId: axisYId), isXAxis: false)
        addDefaultModifiers()
        
    }
    
    private func addSeries() {
        
        let dataCount = 10
        let dataSeries = SCIXyDataSeries(XType: .Float, YType: .Float)
        var i = 1
        while i <= dataCount {
            let x = i
            let y = SCSDataManager.randomize(-1.5, max: 1.5);
            dataSeries.appendX(SCIGeneric(x), y: SCIGeneric(y))
            i += 1
        }
        

        
        let verticalRenderSeries = SCIFastFixedErrorBarsRenderableSeries()
        verticalRenderSeries.dataSeries = dataSeries
        verticalRenderSeries.errorType = SCIErrorBarTypeRelative
        verticalRenderSeries.errorLow = 0.1
        verticalRenderSeries.errorHigh = 0.3
        verticalRenderSeries.xAxisId = axisXId
        verticalRenderSeries.yAxisId = axisYId
        verticalRenderSeries.style.linePen = SCIPenSolid(color: UIColor.init(colorLiteralRed: 70.0/255.0,
            green: 130.0/255.0,
            blue: 180.0/255.0,
            alpha: 1.0), width: 1.0)
        chartSurface.attachRenderableSeries(verticalRenderSeries)
        
        let horizontalRenderSeries = SCIFastFixedErrorBarsRenderableSeries()
        horizontalRenderSeries.errorDirection = SCIErrorBarDirectionHorizontal
        horizontalRenderSeries.errorDataPointWidth = 0.5;
        horizontalRenderSeries.dataSeries = dataSeries
        horizontalRenderSeries.xAxisId = axisXId
        horizontalRenderSeries.yAxisId = axisYId
        horizontalRenderSeries.style.linePen = SCIPenSolid(color: UIColor.redColor(), width: 1.0)
        chartSurface.attachRenderableSeries(horizontalRenderSeries)
        
        let ellipsePointMarker = SCIEllipsePointMarker()
        ellipsePointMarker.drawBorder = true
        ellipsePointMarker.fillBrush = SCIBrushSolid(color: UIColor.init(colorLiteralRed: 70.0/255.0,
            green: 130.0/255.0,
            blue: 180.0/255.0,
            alpha: 1.0))
        ellipsePointMarker.borderPen = SCIPenSolid(color: UIColor.init(colorLiteralRed: 176.0/255.0, green: 196.0/255.0, blue: 222.0/255.0, alpha: 1.0), width: 2.0)
        ellipsePointMarker.height = 15
        ellipsePointMarker.width = 15
        
        let lineRenderSeries = SCIFastLineRenderableSeries()
        lineRenderSeries.xAxisId = axisXId
        lineRenderSeries.yAxisId = axisYId
        lineRenderSeries.dataSeries = dataSeries
        lineRenderSeries.style.pointMarker = ellipsePointMarker
        lineRenderSeries.style.drawPointMarkers = true;
        lineRenderSeries.style.linePen = SCIPenSolid(color: UIColor.init(colorLiteralRed: 176.0/255.0, green: 196.0/255.0, blue: 222.0/255.0, alpha: 1.0), width: 1.0)
        chartSurface.attachRenderableSeries(lineRenderSeries)
        
        chartSurface.invalidateElement()
        
    }
    
}
