//
//  SCSSeriesAppendingTestSciChart.swift
//  SciChartSwiftDemo
//
//  Created by Mykola Hrybeniuk on 6/29/16.
//  Copyright © 2016 SciChart Ltd. All rights reserved.
//

import UIKit
import SciChart

class SCSSeriesAppendingTestSciChart: SCSBaseChartView, SpeedTest {
    
    let randomWalkGenerator = RandomWalkGenerator()
    let paliteColors = [UIColor.init(red: 0.8980, green: 0.2941, blue: 0.1058, alpha: 1),
                        UIColor.init(red: 0.2509, green: 0.5137, blue: 0.7176, alpha: 1),
                        UIColor.init(red: 0.9960, green: 0.6470, blue: 0.0039, alpha: 1)]
    
    
    var delegate: DrawingProtocolDelegate!
    var chartProviderName: String! = "SciChart"
    var parameters: TestParameters!
    var xCount: Int = 0
    var appendCount: Int32 = 0
    var series: [SCIXyDataSeries] = []
    
    // MARK: SpeedTestProtocol
    
    func runTest(testParameters: TestParameters) {
        parameters = testParameters
        addSeries()
    }
    
    func updateChart() {
        delegate.chartExampleStarted!()
        var generalCountPoints : Int32 = 0
        for episod in series {
            var i: Int32 = Int32(xCount);
            while i < xCount + parameters.AppendPoints {
                episod.appendX(SCIGeneric(i), y: SCIGeneric(randomWalkGenerator.next(-0.5, 0.5, true)))
                i += 1
            }
            generalCountPoints = generalCountPoints + episod.count()
        }
        xCount += Int(parameters.AppendPoints)
        
        chartSurface.invalidateElement()
        
        if generalCountPoints > 1000000 {
            stopTest()
        }
        
    }
    
    func stopTest() {
        delegate.processCompleted!()
    }
    
    // MARK: Overrided Functions
    
    override func completeConfiguration() {
        super.completeConfiguration()
        addAxes()
        
    }
    
    // MARK: Private Functions
    
    private func addAxes() {
        
        let axisStyle = generateDefaultAxisStyle()
        
        let axisX = SCSFactoryAxis.createDefaultNumericAxis(withAxisStyle: axisStyle, axisId: axisXId);
        axisX.autoRange = .Always
        axisX.animatedChangeDuration = 1.0/30.0*2
        axisX.animateVisibleRangeChanges = true
        chartSurface.xAxis = axisX
        chartSurface.attachAxis(axisX, isXAxis: true)
        
        let axisY = SCSFactoryAxis.createDefaultNumericAxis(withAxisStyle: axisStyle, axisId: axisYId);
        axisY.autoRange = .Always
        axisY.animatedChangeDuration = 1.0/30.0*2
        axisY.animateVisibleRangeChanges = true
        chartSurface.yAxis = axisY
        chartSurface.attachAxis(axisY, isXAxis: false)
        
        addDefaultModifiers()
        
    }
    
    private func addSeries() {

        var seriesCount: Int32 = 0
        var colorIndex = 0
        
        while seriesCount < parameters.SeriesNumber {
            
            let dataSeries = SCIXyDataSeries(XType: .Double, YType: .Double)
            
            series.append(dataSeries)
            
            dataSeries.dataDistributionCalculator = SCIUserDefinedDistributionCalculator()
            let randomData: [NSArray] = NSArray.init(array: randomWalkGenerator.GetRandomWalkSeries(parameters.PointCount, min: -0.5, max: 0.5, includePrior: true)) as! [NSArray]
            
            var i: Int = 0
            while i < Int(parameters.PointCount) {
                let x = randomData[0][i].doubleValue
                let y = randomData[1][i].doubleValue
                dataSeries.appendX(SCIGeneric(x), y: SCIGeneric(y))
                i += 1
            }
            
            
            let renderSeries = SCIFastLineRenderableSeries()
            renderSeries.dataSeries = dataSeries
            renderSeries.xAxisId = axisXId
            renderSeries.yAxisId = axisYId
            
            if colorIndex >= paliteColors.count {
                colorIndex = 0
            }
            
            
            renderSeries.style.linePen = SCIPenSolid(color: paliteColors[colorIndex], width: 0.5)
            
            chartSurface.attachRenderableSeries(renderSeries)
            

            colorIndex += 1
            seriesCount += 1
            
        }
        xCount += Int(parameters.PointCount)
        
        chartSurface.invalidateElement()
        
    }
    
}



