//
//  SCSStackedMountainChartView.swift
//  SciChartSwiftDemo
//
//  Created by Mykola Hrybeniuk on 6/6/16.
//  Copyright © 2016 SciChart Ltd. All rights reserved.
//

import Foundation
import SciChart

class SCSStackedMountainChartView: SCSBaseChartView {
    
    // MARK: Overrided Functions
    
    override func completeConfiguration() {
        super.completeConfiguration()
        addAxis()
        addDefaultModifiers()
        addDataSeries()
    }
    
    // MARK: Private Methods
    
    private func addAxis() {
        let axisStyle = generateDefaultAxisStyle()
        chartSurface.attachAxis(SCSFactoryAxis.createDefaultDateTimeAxis(withAxisStyle: axisStyle, axisId: axisXId), isXAxis: true)
        chartSurface.attachAxis(SCSFactoryAxis.createDefaultNumericAxis(withAxisStyle: axisStyle, axisId: axisYId), isXAxis: false)
    }
    
    private func addDataSeries() {
        
        let dataSeries = SCIXyDataSeries(XType:.DateTime, YType:.Float)
        dataSeries.dataDistributionCalculator = SCIUserDefinedDistributionCalculator()
        fillDataInto(dataSeries)
        
        let brush = SCIBrushLinearGradient(colorCodeStart: 0x88af9A90,
                                           finish: 0x88af9A43,
                                           direction: .Vertical)
        let renderableSeroiesBottom = createRenderableSeriesWith(brush,
                                                              pen: SCIPenSolid(colorCode: 0xAAffffff, width: 0.5),
                                                              dataSeries: dataSeries)
        
        
        let dataSeries2 = SCIXyDataSeries(XType:.DateTime, YType:.Float)
        dataSeries2.dataDistributionCalculator = SCIUserDefinedDistributionCalculator()
        fillDataInto(dataSeries2)
        
        let brush2 = SCIBrushLinearGradient(colorCodeStart: 0x88ffffff,
                                            finish: 0xDDDBE0E1,
                                            direction: .Vertical)
        let renderableSeroiesTop = createRenderableSeriesWith(brush2,
                                                                 pen: SCIPenSolid(colorCode: 0xFFffffff, width: 0.5),
                                                                 dataSeries: dataSeries2)
        
        
        let stackedGroup = SCIStackedGroupSeries()
        stackedGroup.xAxisId = axisXId
        stackedGroup.yAxisId = axisYId
        stackedGroup.addSeries(renderableSeroiesBottom)
        stackedGroup.addSeries(renderableSeroiesTop)
        chartSurface.attachRenderableSeries(stackedGroup)
        
        chartSurface.invalidateElement()
    
    }
    
    private func createRenderableSeriesWith(brush: SCIBrushLinearGradient, pen: SCIPenSolid, dataSeries: SCIXyDataSeries) ->  SCIStackedMountainRenderableSeries {
        
        let renderableSeries = SCIStackedMountainRenderableSeries()
        renderableSeries.style.areaBrush = brush
        renderableSeries.style.borderPen = pen
        renderableSeries.xAxisId = axisXId
        renderableSeries.yAxisId = axisYId
        renderableSeries.dataSeries = dataSeries
        return renderableSeries;
    }
    
    private func fillDataInto(dataSeries: SCIXyDataSeries) {
        DataManager.loadDataFromFile(dataSeries,
                                     fileName: "FinanceData",
                                     startIndex: 1,
                                     increment: 1,
                                     reverse: true)
    }
    
    
    
}
