//
//  ViewController.swift
//  SciChartSwiftDemo
//
//  Created by Mykola Hrybeniuk on 5/30/16.
//  Copyright © 2016 SciChart Ltd. All rights reserved.
//

import UIKit
import SciChart

class SCSChartViewController: UIViewController, DrawingProtocolDelegate {
    
    var caseTest : TestCase?

    func setupView(viewClass: UIView.Type) {
        let chartView = viewClass.init()
        chartView.frame = self.view.bounds
        chartView.autoresizingMask = [.FlexibleHeight, .FlexibleWidth]
        chartView.translatesAutoresizingMaskIntoConstraints = true
        self.view.addSubview(chartView)
        
        if let _ = chartView as? SpeedTest {
            caseTest = TestCase(version: "", chartUIView: chartView)
            caseTest?.delegate = self
            caseTest?.runTest(self)
        }
        
    }
    
    override func viewDidDisappear(animated: Bool) {
        super.viewDidDisappear(animated)
        if caseTest != nil {
            if !(caseTest?.completed)!{
                caseTest?.processCompleted()
            }
        }
    }
    
    func processCompleted(testCaseData: NSMutableArray!) {
        
        if let results = testCaseData.firstObject as? NSArray where navigationController?.topViewController == self  {
            
            let fps : NSNumber = results[2] as! NSNumber
            let cpu : NSNumber = results[3] as! NSNumber
            let date : NSDate = results[4] as! NSDate
            let formatDate = NSDateFormatter()
            formatDate.dateFormat = "s.SSS"
            let startTime = formatDate.stringFromDate(date)
            
            let descr = String.localizedStringWithFormat("Average FPS: %.2f\nCPU Load: %.2f %%\nStart Time: %@", fps.doubleValue, cpu.doubleValue, startTime)
            let alertController = UIAlertController.init(title: "Finished", message: descr, preferredStyle: .Alert)
            
            alertController.addAction(UIAlertAction(title: "Cancel", style: .Cancel, handler: { (action) in
                
                self.navigationController?.popViewControllerAnimated(true)
                
            }))
                
            alertController.addAction(UIAlertAction(title: "Run Test Again", style: .Default, handler: { (action) in

                self.caseTest?.runTest(self)
                
            }))
            
            presentViewController(alertController, animated: true, completion: nil);
            
        }
        
    }

}

